/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015, 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_sci.c
* Version      : Code Generator for RX231 V1.01.02.05 [24 May 2018]
* Device(s)    : R5F52316AxFM
* Tool-Chain   : CCRX
* Description  : This file implements device driver for SCI module.
* Creation Date: 2020/02/14
***********************************************************************************************************************/

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_sci.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
volatile uint8_t   g_sci12_iic_transmit_receive_flag; /* SCI12 transmit receive flag for I2C */
volatile uint8_t   g_sci12_iic_cycle_flag;            /* SCI12 start stop flag for I2C */
volatile uint8_t   g_sci12_slave_address;             /* SCI12 target slave address */
volatile uint8_t * gp_sci12_tx_address;               /* SCI12 transmit buffer address */
volatile uint16_t  g_sci12_tx_count;                  /* SCI12 transmit data number */
volatile uint8_t * gp_sci12_rx_address;               /* SCI12 receive buffer address */
volatile uint16_t  g_sci12_rx_count;                  /* SCI12 receive data number */
volatile uint16_t  g_sci12_rx_length;                 /* SCI12 receive data length */
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_SCI12_Create
* Description  : This function initializes SCI12.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_SCI12_Create(void)
{
    /* Cancel SCI12 module stop state */
    MSTP(SCI12) = 0U;

    /* Set interrupt priority */
    IPR(SCI12, ERI12) = _0F_SCI_PRIORITY_LEVEL15;

    /* Clear the control register */
    SCI12.SCR.BYTE = 0x00U;

    /* Initialize SSCL and SSDA pins to high impedance */
    SCI12.SIMR3.BYTE = _C0_SCI_SSCL_HIGH_IMPEDANCE | _30_SCI_SSDA_HIGH_IMPEDANCE;

    /* Set up transfer or reception format in SMR and SCMR */
    SCI12.SMR.BYTE = _00_SCI_CLOCK_PCLK;
    SCI12.SCMR.BIT.SMIF = 0U;
    SCI12.SCMR.BIT.SINV = 0U;
    SCI12.SCMR.BIT.SDIR = 1U;

    /* Set bitrate */
    SCI12.BRR = 0x05U;

    /* Set control registers */
    SCI12.SEMR.BYTE = _00_SCI_NOISE_FILTER_DISABLE | _00_SCI_BIT_MODULATION_DISABLE;
    SCI12.SIMR1.BYTE |= _01_SCI_IIC_MODE | _00_SCI12_NONE;
    SCI12.SIMR2.BYTE |= _01_SCI_RX_TX_INTERRUPTS | _02_SCI_SYNCHRONIZATION | _20_SCI_NACK_TRANSMISSION;
    SCI12.SPMR.BYTE = _00_SCI_CLOCK_NOT_INVERTED | _00_SCI_CLOCK_NOT_DELAYED;
    SCI12.SCR.BYTE = _10_SCI_RECEIVE_ENABLE | _20_SCI_TRANSMIT_ENABLE | _40_SCI_RXI_ERI_ENABLE | _80_SCI_TXI_ENABLE |
                    _04_SCI_TEI_INTERRUPT_ENABLE;

    /* Set SSCL12 pin */
    MPC.PE2PFS.BYTE = 0x0CU;
    PORTE.ODR0.BYTE &= 0xDFU;
    PORTE.ODR0.BYTE |= 0x10U;
    PORTE.PMR.BYTE |= 0x04U;

    /* Set SSDA12 pin */
    MPC.PE1PFS.BYTE = 0x0CU;
    PORTE.ODR0.BYTE &= 0xF7U;
    PORTE.ODR0.BYTE |= 0x04U;
    PORTE.PMR.BYTE |= 0x02U;
}
/***********************************************************************************************************************
* Function Name: R_SCI12_Start
* Description  : This function starts SCI12.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_SCI12_Start(void)
{
    /* Clear interrupt flags */
    IR(SCI12, TXI12) = 0U;
    IR(SCI12, RXI12) = 0U;

    /* Enable TXI and STI interrupt */
    IEN(SCI12, TXI12) = 1U;
    IEN(SCI12, TEI12) = 1U;

    /* Enable RXI interrupt */
    IEN(SCI12, RXI12) = 1U;
}
/***********************************************************************************************************************
* Function Name: R_SCI12_Stop
* Description  : This function stops SCI12.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_SCI12_Stop(void)
{
    /* Disable TXI and STI interrupt */
    IEN(SCI12, TXI12) = 0U;
    IEN(SCI12, TEI12) = 0U;

    /* Disable RXI interrupt */
    IEN(SCI12, RXI12) = 0U;

    /* Clear interrupt flags */
    IR(SCI12, TXI12) = 0U;
    IR(SCI12, RXI12) = 0U;
}
/***********************************************************************************************************************
* Function Name: R_SCI12_IIC_Master_Send
* Description  : This function sends IIC12 data to slave device.
* Arguments    : adr -
*                    slave device address
*                tx_buf -
*                    transfer buffer pointer (Not used when transmit data handled by DTC or DMAC)
*                tx_num -
*                    buffer size (Not used when transmit data handled by DTC or DMAC)
* Return Value : None
***********************************************************************************************************************/
void R_SCI12_IIC_Master_Send(uint8_t adr, uint8_t * const tx_buf, uint16_t tx_num)
{
    if (1U > tx_num)
    {
        return;
    }

    gp_sci12_tx_address = tx_buf;
    g_sci12_tx_count = tx_num;
    g_sci12_slave_address = adr;
    g_sci12_iic_transmit_receive_flag = _80_SCI_IIC_TRANSMISSION;
    g_sci12_iic_cycle_flag = _80_SCI_IIC_START_CYCLE;

    /* Disable RXI and ERI interrupt requests */
    SCI12.SCR.BIT.RIE = 0U;

    /* Generate start condition */
    R_SCI12_IIC_StartCondition();
}
/***********************************************************************************************************************
* Function Name: R_SCI12_IIC_Master_Receive
* Description  : This function receives IIC12 data from slave device.
* Arguments    : adr -
*                    slave device address
*                rx_buf -
*                    receive buffer pointer (Not used when receive data handled by DTC or DMAC)
*                rx_num -
*                    buffer size (Not used when receive data handled by DTC or DMAC)
* Return Value : None
***********************************************************************************************************************/
void R_SCI12_IIC_Master_Receive(uint8_t adr, uint8_t * const rx_buf, uint16_t rx_num)
{
    if (1U > rx_num)
    {
        return;
    }

    g_sci12_rx_length = rx_num;
    g_sci12_rx_count = 0;
    gp_sci12_rx_address = rx_buf;
    g_sci12_slave_address = adr;
    g_sci12_iic_transmit_receive_flag = _00_SCI_IIC_RECEPTION;
    g_sci12_iic_cycle_flag = _80_SCI_IIC_START_CYCLE;

    /* Disable RXI and ERI interrupt requests */
    SCI12.SCR.BIT.RIE = 0U;

    /* Generate start condition */
    R_SCI12_IIC_StartCondition();
}
/***********************************************************************************************************************
* Function Name: R_SCI12_IIC_StartCondition
* Description  : This function generate iic start condition.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_SCI12_IIC_StartCondition(void)
{
    SCI12.SIMR3.BYTE = _01_SCI_START_CONDITION_ON | _10_SCI_SSDA_START_RESTART_STOP_CONDITION | 
                       _40_SCI_SSCL_START_RESTART_STOP_CONDITION;
}
/***********************************************************************************************************************
* Function Name: R_SCI12_IIC_StopCondition
* Description  : This function generate iic stop condition.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_SCI12_IIC_StopCondition(void)
{
    SCI12.SIMR3.BYTE = _04_SCI_STOP_CONDITION_ON | _10_SCI_SSDA_START_RESTART_STOP_CONDITION | 
                       _40_SCI_SSCL_START_RESTART_STOP_CONDITION;
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
